import { Button, InputGroup, Label, TextArea } from '@blueprintjs/core'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useLiveQuery } from 'dexie-react-hooks'
import { memo, useCallback, useEffect, useState, type FC } from 'react'
import {
  addEmailTemplate,
  getEmailTemplateByActivityId,
  updateEmailTemplate,
} from '../indexeddb/operations'
import type { EmailTemplate } from '../indexeddb/types'
import EmailAddressSelector from './EmailAddressSelector'

interface EmailTemplateFormProps {
  learningActivityId: number
}

const EmailTemplateForm: FC<EmailTemplateFormProps> = ({
  learningActivityId,
}) => {
  const template = useLiveQuery(
    () => getEmailTemplateByActivityId(learningActivityId),
    [learningActivityId],
    []
  ) as EmailTemplate

  const { notify } = useNotifyContext()

  const [context, setContext] = useState<string>('')
  const [content, setContent] = useState<string>('')
  const [selectedAddressId, setSelectedAddressId] = useState<number>(0)

  useEffect(() => {
    setContext(template?.context || '')
    setContent(template?.content || '')
    setSelectedAddressId(template?.emailAddressId || 0)
  }, [template])

  const handleUpdateButton = useCallback(
    async (newTemplate: EmailTemplate | Omit<EmailTemplate, 'id'>) => {
      try {
        if (template) {
          await updateEmailTemplate({ id: template.id, ...newTemplate })
        } else {
          await addEmailTemplate(newTemplate)
        }
      } catch (err) {
        notify(`Failed to update template: ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify, template]
  )

  return (
    <>
      <EmailAddressSelector
        emailAddressId={selectedAddressId}
        onChange={id => setSelectedAddressId(id)}
      />
      <Label>
        Context
        <InputGroup
          placeholder='Input text'
          value={context}
          onChange={e => setContext(e.target.value)}
        />
      </Label>
      <Label>
        Content
        <TextArea
          value={content}
          style={{
            width: '100%',
            height: '10rem',
            resize: 'none',
            overflowY: 'auto',
          }}
          placeholder='Input text'
          onChange={e => setContent(e.target.value)}
        />
      </Label>
      <Button
        onClick={() =>
          handleUpdateButton({
            learningActivityId,
            context,
            content,
            emailAddressId: selectedAddressId,
          })
        }
        intent='primary'
        icon='edit'
        text='Save changes'
      />
    </>
  )
}

export default memo(EmailTemplateForm)
