import {
  Button,
  Checkbox,
  InputGroup,
  Label,
  TextArea,
} from '@blueprintjs/core'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useLiveQuery } from 'dexie-react-hooks'
import { memo, useCallback, useEffect, useState, type FC } from 'react'
import {
  addToolResponse,
  getToolResponseByActivityId,
  updateToolResponse,
} from '../indexeddb/operations'
import type { ToolResponse } from '../indexeddb/types'
import ToolSelector from './ToolSelector'

interface ToolResponseFormProps {
  learningActivityId: number
}

const ToolResponseForm: FC<ToolResponseFormProps> = ({
  learningActivityId,
}) => {
  const response = useLiveQuery(
    () => getToolResponseByActivityId(learningActivityId),
    [learningActivityId],
    []
  ) as ToolResponse

  const { notify } = useNotifyContext()

  const [parameter, setParameter] = useState<string>('')
  const [content, setContent] = useState<string>('')
  const [isRegex, setIsRegex] = useState<boolean>(false)
  const [selectedToolId, setSelectedToolId] = useState<number>(0)

  useEffect(() => {
    setParameter(response?.parameter || '')
    setContent(response?.content || '')
    setIsRegex(response?.isRegex || false)
    setSelectedToolId(response?.toolId || 0)
  }, [response])

  const handleUpdateButton = useCallback(
    async (newResponse: ToolResponse | Omit<ToolResponse, 'id'>) => {
      try {
        if (response) {
          await updateToolResponse({ id: response.id, ...newResponse })
        } else {
          await addToolResponse(newResponse)
        }
      } catch (err) {
        notify(`Failed to update tool response: ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify, response]
  )

  return (
    <>
      <ToolSelector
        toolId={selectedToolId}
        onChange={id => setSelectedToolId(id)}
      />
      <Label>
        Parameter
        <InputGroup
          placeholder='Input text'
          value={parameter}
          onChange={e => setParameter(e.target.value)}
        />
      </Label>
      <Checkbox
        label='Is parameter regex?'
        checked={isRegex}
        onChange={e => setIsRegex(e.target.checked)}
      />
      <Label>
        Response
        <TextArea
          value={content}
          style={{
            width: '100%',
            height: '10rem',
            resize: 'none',
            overflowY: 'auto',
          }}
          placeholder='Input text'
          onChange={e => setContent(e.target.value)}
        />
      </Label>
      <Button
        onClick={() =>
          handleUpdateButton({
            learningActivityId,
            parameter,
            content,
            isRegex,
            toolId: selectedToolId,
          })
        }
        intent='primary'
        icon='edit'
        text='Save changes'
      />
    </>
  )
}

export default memo(ToolResponseForm)
