import { Button, Card, Overlay2 } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { EmailThread } from '@inject/graphql/fragments/EmailThread.generated'
import type { FileInfo } from '@inject/graphql/fragments/FileInfo.generated'
import type { FC } from 'react'
import { useCallback, useEffect, useState } from 'react'
import InstructorEmailForm from '../EmailForm/InstructorEmailForm'
import TraineeEmailForm from '../EmailForm/TraineeEmailForm'
import useFormState from '../EmailForm/useFormState'

const card = css`
  height: 50vh;
  width: 40vw;
  padding: 0 !important;
  display: flex;
  flex-direction: column;
  right: 1vw;
  bottom: 0;
`

export const OPEN_COMPOSE_EVENT_TYPE = 'openCompose'
export const OPEN_REPLY_EVENT_TYPE = 'openReply'

type EmailFormOverlayProps = {
  teamId: string
  exerciseId: string
} & (
  | { emailForm: 'trainee'; teamAddress: string }
  | { emailForm: 'instructor'; teamAddress?: undefined }
)

interface ComposeInitialValues {
  content?: string
  fileInfo?: FileInfo
  subject?: string
}

type OpenComposeEventPayload = ComposeInitialValues | undefined

interface OpenReplyEventPayload {
  emailThread: EmailThread
}

const EmailFormOverlay: FC<EmailFormOverlayProps> = ({
  teamId,
  emailForm,
  teamAddress,
  exerciseId,
}) => {
  const [open, setOpen] = useState(false)
  const [emailThread, setEmailThread] = useState<EmailThread | undefined>()

  const formState = useFormState({
    teamAddress: emailForm === 'trainee' ? teamAddress : undefined,
    inInstructor: emailForm === 'instructor',
    teamId,
  })
  const { reload } = formState

  const handleOpenCompose = useCallback(
    (event: CustomEvent<OpenComposeEventPayload>) => {
      setOpen(true)
      setEmailThread(undefined)

      if (event.detail) {
        const { content, fileInfo, subject } = event.detail
        reload({
          emailThreadId: undefined,
          loadDraft: false,
          content,
          fileInfo,
          subject,
        })
      } else {
        reload({ emailThreadId: undefined, loadDraft: true })
      }
    },
    [reload]
  )
  useEffect(() => {
    window.addEventListener(
      OPEN_COMPOSE_EVENT_TYPE,
      handleOpenCompose as EventListener
    )
    return () => {
      window.removeEventListener(
        OPEN_COMPOSE_EVENT_TYPE,
        handleOpenCompose as EventListener
      )
    }
  }, [handleOpenCompose])

  const handleOpenReply = useCallback(
    (event: CustomEvent<OpenReplyEventPayload>) => {
      setOpen(true)
      setEmailThread(event.detail.emailThread)
      reload({ emailThreadId: event.detail.emailThread.id, loadDraft: true })
    },
    [reload]
  )
  useEffect(() => {
    window.addEventListener(
      OPEN_REPLY_EVENT_TYPE,
      handleOpenReply as EventListener
    )
    return () => {
      window.removeEventListener(
        OPEN_REPLY_EVENT_TYPE,
        handleOpenReply as EventListener
      )
    }
  }, [handleOpenReply])

  useEffect(() => {
    setOpen(false)
  }, [teamId])

  return (
    <Overlay2
      isOpen={open}
      autoFocus={false}
      canEscapeKeyClose={false}
      canOutsideClickClose={false}
      enforceFocus={false}
      hasBackdrop={false}
      transitionDuration={0}
    >
      <Card className={card} compact elevation={4}>
        <Button
          minimal
          icon='cross'
          style={{ marginLeft: 'auto' }}
          onClick={() => setOpen(false)}
        />
        {emailForm === 'trainee' ? (
          <TraineeEmailForm
            onSuccess={() => setOpen(false)}
            teamId={teamId}
            exerciseId={exerciseId}
            emailThread={emailThread}
            formState={formState}
          />
        ) : (
          <InstructorEmailForm
            onSuccess={() => setOpen(false)}
            teamId={teamId}
            exerciseId={exerciseId}
            emailThread={emailThread}
            formState={formState}
          />
        )}
      </Card>
    </Overlay2>
  )
}

export default EmailFormOverlay
