import { InputGroup, MenuItem, Tooltip } from '@blueprintjs/core'
import type { ItemRenderer } from '@blueprintjs/select'
import { Select } from '@blueprintjs/select'
import type { EmailParticipant } from '@inject/graphql/fragments/EmailParticipant.generated'
import type { Dispatch, SetStateAction } from 'react'
import { useCallback, type FC } from 'react'
import type { ExtendedItemRenderer } from '../typing'

interface SenderSelectorProps {
  emailContacts: EmailParticipant[]
  senderList: string[]
  senderAddress: string
  setSenderAddress: Dispatch<SetStateAction<string>>
}

const SenderSelector: FC<SenderSelectorProps> = ({
  emailContacts,
  senderList,
  senderAddress,
  setSenderAddress,
}) => {
  const itemRenderer: ExtendedItemRenderer<string> = useCallback(
    (address, { handleClick, handleFocus, modifiers, hasTooltip }) => (
      <MenuItem
        icon={hasTooltip ? 'small-info-sign' : undefined}
        active={modifiers.active}
        disabled={modifiers.disabled}
        onClick={handleClick}
        onFocus={handleFocus}
        roleStructure='listoption'
        selected={senderAddress === address}
        shouldDismissPopover={false}
        text={address}
        key={address}
      />
    ),
    [senderAddress]
  )

  const tooltipRenderer: ItemRenderer<string> = useCallback(
    (address, props) => {
      const tooltipContent = emailContacts.find(
        contact => contact.address === address
      )?.definitionAddress?.description

      return tooltipContent ? (
        <Tooltip content={tooltipContent} targetTagName='div' key={address}>
          {itemRenderer(address, { ...props, hasTooltip: true })}
        </Tooltip>
      ) : (
        itemRenderer(address, props)
      )
    },
    [emailContacts, itemRenderer]
  )

  return (
    <Select<string>
      itemRenderer={tooltipRenderer}
      items={senderList}
      onItemSelect={item => setSenderAddress(item)}
      popoverProps={{
        minimal: true,
      }}
      filterable={false}
      popoverContentProps={{
        style: { maxHeight: '50vh', overflowY: 'auto', overflowX: 'hidden' },
      }}
    >
      <InputGroup
        value={
          senderList.length > 0
            ? senderAddress
            : 'Select at least one non-team recipient'
        }
        fill
        disabled={senderList.length === 0}
      />
    </Select>
  )
}

export default SenderSelector
