import ErrorMessage from '@/components/ErrorMessage'
import ExerciseListComponent from '@/components/ExerciseList'
import { FormGroup, NonIdealState, Spinner, Switch } from '@blueprintjs/core'
import type { Exercise } from '@inject/graphql/fragments/Exercise.generated'
import { useDeleteExercise } from '@inject/graphql/mutations/DeleteExercise.generated'
import { useStartExercise } from '@inject/graphql/mutations/StartExercise.generated'
import { useStopExercise } from '@inject/graphql/mutations/StopExercise.generated'
import { GetExercisesDocument } from '@inject/graphql/queries/GetExercises.generated'
import { GetRunningExercisesDocument } from '@inject/graphql/queries/GetRunningExercises.generated'
import useExercisesSubscription from '@inject/graphql/utils/useExercisesSubscription'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import notEmpty from '@inject/shared/utils/notEmpty'
import { useSessionStorageState } from 'ahooks'
import { useCallback, useMemo } from 'react'

const ExerciseList = () => {
  const { data, loading, error } = useExercisesSubscription()

  const [setting, setSetting] = useSessionStorageState<{
    showFinished: boolean
  }>('exerciseList', {
    deserializer: JSON.parse,
    serializer: JSON.stringify,
    defaultValue: {
      showFinished: true,
    },
  })
  const { showFinished } = setting || { showFinished: true }
  const exercises = useMemo(
    () =>
      (data?.exercises ?? [])
        .filter(notEmpty)
        .filter(
          exercise => showFinished || (!showFinished && !exercise.finished)
        ),
    [data, showFinished]
  )

  const [mutateStart, { loading: loadingStart }] = useStartExercise()
  const [mutatePause, { loading: loadingPause }] = useStopExercise()
  const [mutateRemove, { loading: loadingRemove }] = useDeleteExercise()

  const { notify } = useNotifyContext()

  const runningAny = useMemo(
    () => exercises.some(exercise => exercise.running),
    [exercises]
  )

  const handleStart = useCallback(
    (exercise: Exercise) => {
      mutateStart({
        variables: {
          id: exercise.id,
        },
        refetchQueries: [GetExercisesDocument, GetRunningExercisesDocument],
      }).then(() => {
        notify(`Exercise ${exercise.id} started`)
      })
    },
    [mutateStart, notify]
  )
  const handlePause = useCallback(
    (exercise: Exercise) => {
      mutatePause({
        variables: {
          id: exercise.id,
        },
        refetchQueries: [GetExercisesDocument, GetRunningExercisesDocument],
      }).then(() => {
        notify(`Exercise ${exercise.id} paused`)
      })
    },
    [mutatePause, notify]
  )
  const handleRemove = useCallback(
    (exercise: Exercise) => {
      mutateRemove({
        variables: {
          id: exercise.id,
        },
        refetchQueries: [GetExercisesDocument, GetRunningExercisesDocument],
      }).then(() => {
        notify(`Exercise ${exercise.id} removed`)
      })
    },
    [mutateRemove, notify]
  )

  const startButtonProps = useCallback(
    (exercise: Exercise) => ({
      loading: loadingStart,
      onClick: () => handleStart(exercise),
      disabled: runningAny,
      title: runningAny ? 'Only one exercise can run at a time' : undefined,
    }),
    [handleStart, loadingStart, runningAny]
  )
  const removeButtonProps = useCallback(
    (exercise: Exercise) => ({
      loading: loadingRemove,
      onClick: () => handleRemove(exercise),
    }),
    [handleRemove, loadingRemove]
  )
  const pauseButtonProps = useCallback(
    (exercise: Exercise) => ({
      loading: loadingPause,
      onClick: () => handlePause(exercise),
    }),
    [handlePause, loadingPause]
  )

  const exerciseList = useMemo(
    () =>
      exercises.length === 0 ? (
        <NonIdealState
          icon='low-voltage-pole'
          title='No exercise found'
          description='There are no exercises to display'
        />
      ) : (
        <ExerciseListComponent
          exercises={exercises}
          type='managing'
          details={{
            startButtonProps,
            removeButtonProps,
            pauseButtonProps,
          }}
        />
      ),
    [exercises, pauseButtonProps, removeButtonProps, startButtonProps]
  )

  if (loading) {
    return <Spinner />
  } else if (error) {
    return (
      <ErrorMessage>
        <h1>Error occurred!</h1>
        <p>{error.message}</p>
      </ErrorMessage>
    )
  } else if (!data || !data.exercises) {
    return (
      <NonIdealState
        icon='low-voltage-pole'
        title='No data'
        description='Please wait for the data to come in'
      />
    )
  }

  return (
    <div>
      <FormGroup>
        <Switch
          checked={showFinished}
          label='Show finished exercises'
          onChange={() =>
            setSetting(prev => ({ ...prev, showFinished: !showFinished }))
          }
        />
      </FormGroup>

      {exerciseList}
    </div>
  )
}

export default ExerciseList
