import UsersSelection from '@/users/UsersSelection'
import { Button, NonIdealState, Section, SectionCard } from '@blueprintjs/core'
import { useAddDefinitionAccess } from '@inject/graphql/mutations/AddDefinitionAccess.generated'
import { useRemoveDefinitionAccess } from '@inject/graphql/mutations/RemoveDefinitionAccess.generated'
import {
  GetDefinitionDocument,
  useGetDefinition,
} from '@inject/graphql/queries/GetDefinition.generated'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { FC } from 'react'
import { useCallback, useState } from 'react'
import UserTableSelection from '../UserTableSelection'

interface DefinitionAssignmentProps {
  definitionId: string
}

const DefinitionAssignment: FC<DefinitionAssignmentProps> = ({
  definitionId,
}) => {
  const [addDefinitionAccess] = useAddDefinitionAccess()
  const [removeDefinitionAccess] = useRemoveDefinitionAccess()
  const { data, refetch } = useGetDefinition({
    variables: {
      definitionId,
    },
    fetchPolicy: 'network-only',
  })
  const { notify } = useNotifyContext()
  const [listView, setListView] = useState(true)

  const handleAssignButton = useCallback(
    (ids: string[]) => {
      addDefinitionAccess({
        variables: {
          definitionId,
          userIds: ids,
        },
        refetchQueries: [GetDefinitionDocument],
      })
        .then(() => {
          notify('Instructors assigned to the defition')
        })
        .catch(() => {
          notify('Definition access asignment failed', {
            intent: 'danger',
          })
        })
    },
    [addDefinitionAccess, notify]
  )

  const handleRemoveButton = useCallback(
    (ids: string[]) => {
      removeDefinitionAccess({
        variables: {
          definitionId,
          userIds: ids,
        },
        refetchQueries: [GetDefinitionDocument],
      })
        .then(() => {
          notify('Instructors removed from the defition')
        })
        .catch(() => {
          notify('Definition access removal failed', {
            intent: 'danger',
          })
        })
    },
    [removeDefinitionAccess, notify]
  )

  if (!data || !data.definition) {
    return (
      <NonIdealState
        icon='low-voltage-pole'
        description='Unable to display users'
      />
    )
  }

  return (
    <Section
      title={`Assign instructors - definition ${data?.definition.name}`}
      rightElement={
        listView ? (
          <Button
            onClick={() => {
              refetch()
            }}
            icon='reset'
            minimal
          >
            Reload
          </Button>
        ) : undefined
      }
    >
      <SectionCard>
        {listView ? (
          <UsersSelection
            users={data?.definition.userSet?.filter(notEmpty) || []}
            onRemove={(ids: string[]) => handleRemoveButton(ids)}
            onAssign={() => setListView(prev => !prev)}
            assignTitle='Add'
            removeTitle='Remove'
          />
        ) : (
          <UserTableSelection
            onBack={() => setListView(prev => !prev)}
            onAssign={(ids: string[]) => handleAssignButton(ids)}
            assignTitle='Add selected'
          />
        )}
      </SectionCard>
    </Section>
  )
}

export default DefinitionAssignment
