import UsersSelection from '@/users/UsersSelection'
import { Button, NonIdealState, Section, SectionCard } from '@blueprintjs/core'
import { useAssignInstructorsToExercise } from '@inject/graphql/mutations/AssignInstructorsToExercise.generated'
import { useAssignUsersToTeam } from '@inject/graphql/mutations/AssignUsersToTeam.generated'
import { useRemoveInstructorsFromExercise } from '@inject/graphql/mutations/RemoveInstructorsFromExercise.generated'
import { useRemoveUsersFromTeam } from '@inject/graphql/mutations/RemoveUsersFromTeam.generated'
import {
  GetExerciseDocument,
  useGetExercise,
} from '@inject/graphql/queries/GetExercise.generated'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { FC } from 'react'
import { useCallback, useState } from 'react'
import UserTableSelection from '../UserTableSelection'

interface ExerciseAssignmentProps {
  exerciseId: string
}

const ExerciseAssignment: FC<ExerciseAssignmentProps> = ({ exerciseId }) => {
  const [assignUsersToTeam] = useAssignUsersToTeam()
  const [removeUsersFromTeam] = useRemoveUsersFromTeam()
  const [assignInstructorsToExercise] = useAssignInstructorsToExercise()
  const [removeInstructorsFromExercise] = useRemoveInstructorsFromExercise()
  const { data, refetch } = useGetExercise({
    variables: {
      exerciseId,
    },
    fetchPolicy: 'network-only',
  })
  const { notify } = useNotifyContext()
  const [listView, setListView] = useState(true)
  const [currentTeam, setCurrentTeam] = useState<string | null>(null)

  const handleAssignToTeamButton = useCallback(
    (ids: string[]) => {
      assignUsersToTeam({
        variables: {
          teamId: currentTeam || '',
          userIds: ids,
        },
        refetchQueries: [GetExerciseDocument],
      })
        .then(() => {
          notify('Users assigned to team')
        })
        .catch(() => {
          notify('Users team assignment failed', {
            intent: 'danger',
          })
        })
    },
    [currentTeam, assignUsersToTeam, notify]
  )

  const handleRemoveFromTeamButton = useCallback(
    (teamId: string, ids: string[]) => {
      removeUsersFromTeam({
        variables: {
          teamId,
          userIds: ids,
        },
        refetchQueries: [GetExerciseDocument],
      })
        .then(() => {
          notify('Users removed from team')
        })
        .catch(() => {
          notify('Users removal from team failed', {
            intent: 'danger',
          })
        })
    },
    [removeUsersFromTeam, notify]
  )

  const handleAssignToExerciseButton = useCallback(
    (ids: string[]) => {
      assignInstructorsToExercise({
        variables: {
          exerciseId,
          userIds: ids,
        },
        refetchQueries: [GetExerciseDocument],
      })
        .then(() => {
          notify('Instructors assigned to the exercise')
        })
        .catch(() => {
          notify('Instructors exercise assignment failed', {
            intent: 'danger',
          })
        })
    },
    [assignInstructorsToExercise, notify]
  )

  const handleRemoveFromExerciseButton = useCallback(
    (ids: string[]) => {
      removeInstructorsFromExercise({
        variables: {
          exerciseId,
          userIds: ids,
        },
        refetchQueries: [GetExerciseDocument],
      })
        .then(() => {
          notify('Instructors removed from the exercise')
        })
        .catch(() => {
          notify('Instructors removal from exercise failed', {
            intent: 'danger',
          })
        })
    },
    [removeInstructorsFromExercise, notify]
  )

  if (!data || !data.exerciseId) {
    return (
      <NonIdealState
        icon='low-voltage-pole'
        description='Unable to display participants'
      />
    )
  }

  return (
    <Section
      title={`Participants - exercise ${data?.exerciseId.id}`}
      rightElement={
        <Button
          onClick={() => {
            refetch()
          }}
          icon='reset'
          minimal
        >
          Reload
        </Button>
      }
    >
      <SectionCard>
        {listView ? (
          <>
            <Section title='Instructors'>
              <SectionCard>
                <UsersSelection
                  users={data?.exerciseId.userSet?.filter(notEmpty) || []}
                  onRemove={ids => handleRemoveFromExerciseButton(ids)}
                  onAssign={() => {
                    setListView(prev => !prev)
                    setCurrentTeam(null)
                  }}
                  assignTitle='Add'
                  removeTitle='Remove'
                />
              </SectionCard>
            </Section>
            {data?.exerciseId.teams.map(team => (
              <Section
                key={team.id}
                title={`${team.name}${team.role ?? ` - ${team.role}`}`}
                style={{ marginTop: '1rem' }}
              >
                <SectionCard>
                  <UsersSelection
                    users={team.userSet?.filter(notEmpty) || []}
                    onRemove={(ids: string[]) =>
                      handleRemoveFromTeamButton(team.id, ids)
                    }
                    onAssign={() => {
                      setListView(prev => !prev)
                      setCurrentTeam(team.id)
                    }}
                    assignTitle='Add'
                    removeTitle='Remove'
                  />
                </SectionCard>
              </Section>
            ))}
          </>
        ) : (
          <UserTableSelection
            onBack={() => setListView(prev => !prev)}
            onAssign={(ids: string[]) =>
              currentTeam === null
                ? handleAssignToExerciseButton(ids)
                : handleAssignToTeamButton(ids)
            }
            assignTitle='Add selected'
          />
        )}
      </SectionCard>
    </Section>
  )
}

export default ExerciseAssignment
