import { useNavigate } from '@/router'
import { Button, NonIdealState, Section, SectionCard } from '@blueprintjs/core'
import { css } from '@emotion/css'
import { useGetUser } from '@inject/graphql/queries/GetUser.generated'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { FC } from 'react'
import UserDetailRow from './UserDetailRow'
import UserDetailSetting from './UserDetailSetting'

const detail = css`
  display: grid;
  grid-template-columns: 50% 50%;
  grid-column-gap: 10px;
`

export const detailRow = css`
  padding-bottom: 0.75rem;
`

export const detailRowName = css`
  font-weight: bolder;
  padding-right: 0.5rem;
`

interface ListItemProps {
  name: string
  onClick: () => void
}

const ListItem: FC<ListItemProps> = ({ name, onClick }) => (
  <div>
    {name}
    <Button minimal icon='annotation' onClick={onClick} />
  </div>
)

interface UserDetailProps {
  userId: string
}

const UserDetail: FC<UserDetailProps> = ({ userId }) => {
  const nav = useNavigate()

  const { data: userData, refetch } = useGetUser({
    variables: {
      userId,
    },
    fetchPolicy: 'network-only',
  })

  if (!userData || !userData.user) {
    return (
      <NonIdealState
        icon='low-voltage-pole'
        description='Unable to display user'
      />
    )
  }

  return (
    <Section
      title={`User detail - ${userId}`}
      rightElement={
        <Button
          onClick={() => {
            refetch()
          }}
          icon='reset'
          minimal
        >
          Reload
        </Button>
      }
    >
      <SectionCard>
        <div className={detail}>
          <div>
            <UserDetailRow name={'Username'} value={userData.user.username} />
            <UserDetailRow
              name={'First name'}
              value={userData.user.firstName}
            />
            <UserDetailRow name={'Last name'} value={userData.user.lastName} />
            <UserDetailRow
              name={'Date joined'}
              value={userData.user.dateJoined}
            />
            <UserDetailRow
              name={'Last login'}
              value={userData.user.lastLogin}
            />
            <UserDetailRow name={'Staff'} value={userData.user.isStaff} />
            <UserDetailRow
              name={'Superuser'}
              value={userData.user.isSuperuser}
            />
            <UserDetailRow
              name={'Tags'}
              value={userData.user.tags.map(tag => tag.name).join(', ')}
            />
            <UserDetailSetting user={userData.user} />
          </div>
          <div>
            {(userData.user.exercises?.length || 0) +
              (userData.user.teams?.length || 0) !==
              0 && (
              <div>
                <span className={detailRowName}>Exercises:</span>
                {userData.user.exercises &&
                  userData.user.exercises.filter(notEmpty).map(exercise => (
                    <ListItem
                      key={exercise.id}
                      name={`Exercise ${exercise.id} - instructor `}
                      onClick={() =>
                        nav('/users/exercise/:exerciseId', {
                          params: { exerciseId: exercise.id || '' },
                          state: { fromUserDetail: true },
                        })
                      }
                    />
                  ))}
                {userData.user.teams &&
                  userData.user.teams.filter(notEmpty).map(team => (
                    <ListItem
                      key={team.id}
                      name={`Exercise ${team.exercise.id} - ${team.name + (team.role ?? `, ${team.role}`)} `}
                      onClick={() =>
                        nav('/users/exercise/:exerciseId', {
                          params: { exerciseId: team.exercise.id },
                          state: { fromUserDetail: true },
                        })
                      }
                    />
                  ))}
              </div>
            )}
            {userData.user.definitions &&
              userData.user.definitions.length !== 0 && (
                <div>
                  <span className={detailRowName}>Definitions:</span>
                  {userData.user.definitions
                    .filter(notEmpty)
                    .map(definition => (
                      <ListItem
                        key={definition.id}
                        name={`Definition ${definition.id} `}
                        onClick={() =>
                          definition.id &&
                          nav('/users/definition/:definitionId', {
                            params: { definitionId: definition.id.toString() },
                            state: { fromUserDetail: true },
                          })
                        }
                      />
                    ))}
                </div>
              )}
          </div>
        </div>
      </SectionCard>
    </Section>
  )
}

export default UserDetail
