import Filters from '@/components/Filters'
import type { Column, ValueType } from '@/components/SortableTable'
import SortableTable from '@/components/SortableTable'
import { Button, Checkbox, Divider, InputGroup } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { Tag } from '@inject/graphql/fragments/Tag.generated'
import type { User } from '@inject/graphql/fragments/User.generated'
import { useGetGroups } from '@inject/graphql/queries/GetGroups.generated'
import { useGetTags } from '@inject/graphql/queries/GetTags.generated'
import { useGetUsers } from '@inject/graphql/queries/GetUsers.generated'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { FC } from 'react'
import React, { useEffect, useMemo, useState } from 'react'
import UsersUploader from '../UsersUploader'
import { accountStatuses, formatValue } from '../utilities'

const filterTab = css`
  & div {
    padding-right: 0.5rem;
  }
`

const selectedRow = css`
  background: rgba(143, 153, 168, 0.15);
`

export interface UserType extends User {
  [key: string]: UserValueType
}

export type UserValueType = boolean | Array<object | null> | ValueType

type UserObject = Record<keyof UserType, undefined>

interface FilterGroupProps {
  items: string[]
  displayState: boolean[]
  setDisplayState: React.Dispatch<React.SetStateAction<boolean[]>>
}

const FilterGroup = ({
  items,
  displayState,
  setDisplayState,
}: FilterGroupProps) => (
  <>
    {items.map((item, i) => (
      <Checkbox
        key={item}
        label={item}
        checked={displayState[i]}
        onChange={() =>
          setDisplayState(prev => [
            ...prev.slice(0, i),
            !prev[i],
            ...prev.slice(i + 1),
          ])
        }
      />
    ))}
  </>
)

interface UserTableProps {
  allowAddingUsers?: boolean
  onClick: (id: string) => void
  selectedUsers?: string[]
}

const UserTable: FC<UserTableProps> = ({
  allowAddingUsers,
  onClick,
  selectedUsers,
}) => {
  const { data: usersData, refetch } = useGetUsers({
    fetchPolicy: 'network-only',
  })
  const { data: groupsData } = useGetGroups({
    fetchPolicy: 'cache-first',
  })
  const { data: tagsData } = useGetTags({
    fetchPolicy: 'network-only',
  })

  const userPropertiesObject: UserObject = {
    id: undefined,
    username: undefined,
    firstName: undefined,
    lastName: undefined,
    group: undefined,
    isActive: undefined,
    dateJoined: undefined,
    isStaff: undefined,
    isSuperuser: undefined,
    lastLogin: undefined,
    tags: undefined,
  }
  const userProperties = Object.keys(userPropertiesObject) as string[]
  const [displayProperty, setDisplayProperty] = useState(
    userProperties.map((_userProperty, i) => 0 < i && i < 7)
  )
  const displayedProperties = userProperties.filter(
    (_userProperty, i) => displayProperty[i]
  )

  const userGroups = useMemo(
    () => groupsData?.groups?.filter(notEmpty).map(group => group.name) || [],
    [groupsData]
  )
  const [displayUserGroup, setDisplayUserGroup] = useState(
    new Array(userGroups.length).fill(true)
  )
  const displayedUserGroups = userGroups.filter(
    (_userGroup, i) => displayUserGroup[i]
  )
  useEffect(() => {
    setDisplayUserGroup(new Array(userGroups.length).fill(true))
  }, [groupsData])

  const [displayAccountStatus, setDisplayAccountStatus] = useState(
    new Array(accountStatuses.length).fill(true)
  )
  const displayedAccountStatuses = accountStatuses.filter(
    (_accountStatus, i) => displayAccountStatus[i]
  )

  const tags = useMemo(
    () => tagsData?.tags?.filter(notEmpty).map(tag => tag.name) || [],
    [tagsData]
  )
  const [displayTag, setDisplayTag] = useState(
    new Array(tags.length).fill(false)
  )
  const displayedTags = tags.filter((_tag, i) => displayTag[i])
  useEffect(() => {
    setDisplayTag(new Array(userGroups.length).fill(false))
  }, [groupsData])

  const [username, setUsername] = useState<string>('')
  const [firstName, setFirstName] = useState<string>('')
  const [lastName, setLastName] = useState<string>('')

  const columns: Column[] = [
    ...displayedProperties.map(property => ({
      name: property,
      style: { width: `${100 / displayedProperties.length}%` },
    })),
  ]

  const rows = (usersData?.users || [])
    .filter(notEmpty)
    .filter(
      (user: User) =>
        user !== null &&
        displayedUserGroups.includes(user.group || '') &&
        displayedAccountStatuses.includes(formatValue(user.isActive)) &&
        (user.username || '').includes(username) &&
        (user.firstName || '').includes(firstName) &&
        (user.lastName || '').includes(lastName) &&
        displayedTags.every(tag =>
          user.tags
            .filter(notEmpty)
            .map(tag => tag.name)
            .includes(tag)
        )
    )
    .filter(notEmpty)
    .map((user: User) => ({
      columns,
      values: columns.map(column => {
        const value = (user as UserType)[column.name]
        if (value instanceof Array) {
          if (!value) {
            return ''
          }
          return value.map(tag => (tag as Tag).name).join(', ')
        }
        return value as ValueType
      }),
      onClick: () => onClick(user.id),
      class: selectedUsers?.includes(user.id) ? selectedRow : undefined,
    }))

  const filters = (
    <FilterGroup
      items={userProperties}
      displayState={displayProperty}
      setDisplayState={setDisplayProperty}
    />
  )

  const groupFilters = (
    <FilterGroup
      items={userGroups}
      displayState={displayUserGroup}
      setDisplayState={setDisplayUserGroup}
    />
  )

  const accountStatusFilters = (
    <FilterGroup
      items={accountStatuses}
      displayState={displayAccountStatus}
      setDisplayState={setDisplayAccountStatus}
    />
  )

  const tagFilters = (
    <FilterGroup
      items={tags}
      displayState={displayTag}
      setDisplayState={setDisplayTag}
    />
  )

  return (
    <div style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
      <div
        style={{
          display: 'flex',
          justifyContent: 'space-between',
          marginBottom: '0.5rem',
        }}
      >
        <Filters
          heading='Displayed properties'
          content={filters}
          position='bottom-left'
          onSelectAll={() => setDisplayProperty(prev => prev.map(() => true))}
          onDeselectAll={() =>
            setDisplayProperty(prev => prev.map(() => false))
          }
        />
        <div>
          <Button
            onClick={() => {
              refetch()
            }}
            icon='reset'
          >
            Reload
          </Button>
          {allowAddingUsers && <UsersUploader />}
        </div>
      </div>
      <div style={{ display: 'flex' }} className={filterTab}>
        <Filters
          heading='User groups'
          content={groupFilters}
          position='bottom-left'
          onSelectAll={() => setDisplayUserGroup(prev => prev.map(() => true))}
          onDeselectAll={() =>
            setDisplayUserGroup(prev => prev.map(() => false))
          }
        />
        <Filters
          heading='Active'
          content={accountStatusFilters}
          position='bottom-left'
          onSelectAll={() =>
            setDisplayAccountStatus(prev => prev.map(() => true))
          }
          onDeselectAll={() =>
            setDisplayAccountStatus(prev => prev.map(() => false))
          }
        />
        <Filters
          heading='Tags'
          content={tagFilters}
          position='bottom-left'
          onSelectAll={() => setDisplayTag(prev => prev.map(() => true))}
          onDeselectAll={() => setDisplayTag(prev => prev.map(() => false))}
        />
        <InputGroup
          placeholder='Username'
          value={username}
          onChange={e => setUsername(e.target.value)}
        />
        <InputGroup
          placeholder='First name'
          value={firstName}
          onChange={e => setFirstName(e.target.value)}
        />
        <InputGroup
          placeholder='Last name'
          value={lastName}
          onChange={e => setLastName(e.target.value)}
        />
      </div>
      <Divider />
      <div style={{ flex: 1 }}>
        <SortableTable
          columns={columns}
          rows={rows}
          formatValue={formatValue}
        />
      </div>
    </div>
  )
}

UserTable.defaultProps = {
  allowAddingUsers: true,
}

export default UserTable
