import { useEffect } from 'react'
import { useStartExercise } from '../mutations/StartExercise.generated'
import { useStopExercise } from '../mutations/StopExercise.generated'
import { useGetExercise } from '../queries/GetExercise.generated'
import { useGetExerciseLoopStatus } from '../queries/GetExerciseLoopStatus.generated'
import type { ExerciseLoopRunningSubscriptionResult } from '../subscriptions/ExerciseLoopStatus.generated'
import { ExerciseLoopRunningDocument } from '../subscriptions/ExerciseLoopStatus.generated'

const useExerciseSubscriptionStatus = (exerciseId?: string) => {
  const {
    data: dataExerciseLoop,
    subscribeToMore,
    refetch: refetchExerciseLoop,
  } = useGetExerciseLoopStatus({
    fetchPolicy: 'network-only',
    nextFetchPolicy: 'cache-first',
    variables: {
      exerciseId: exerciseId || '',
    },
    skip: !exerciseId,
  })
  const { data: dataExercise } = useGetExercise({
    variables: { exerciseId: exerciseId || '' },
    skip: !exerciseId,
  })
  const [stopExercise] = useStopExercise()
  const [startExercise] = useStartExercise()

  useEffect(() => {
    if (exerciseId) {
      return subscribeToMore({
        document: ExerciseLoopRunningDocument,
        variables: { exerciseId },
        updateQuery: (prev, { subscriptionData }) => {
          const data = subscriptionData as ExerciseLoopRunningSubscriptionResult
          if (!data.data) return prev
          return {
            ...prev,
            exerciseLoopRunning:
              data.data.exerciseLoopRunning?.exerciseLoopRunning ?? null,
          }
        },
      })
    }
  }, [subscribeToMore, exerciseId])

  /**
   * During a backend server outage, the exercise loop of a running exercise
   * will be stopped, but the exercise will still be marked as running
   *
   * This useEffect resolves this inconsistency by restarting the exercise and
   * refetching the exercise loop status; only one client can do this at a time
   *
   * Note that the issue will persist if the exercise is not subscribed to at
   * the moment of the backend server failure
   */
  useEffect(() => {
    const asyncEffect = async () => {
      while (
        dataExerciseLoop?.exerciseLoopRunning === false &&
        dataExercise?.exerciseId?.running === true &&
        exerciseId
      ) {
        const lock = localStorage.getItem('backendOutageFixLock')
        if (lock) {
          await new Promise(resolve => setTimeout(resolve, 1000))
          await refetchExerciseLoop()
          return
        }

        localStorage.setItem('backendOutageFixLock', 'true')
        stopExercise({
          variables: { id: exerciseId },
          onCompleted: () => {
            startExercise({
              variables: { id: exerciseId },
              onCompleted: () => {
                localStorage.removeItem('backendOutageFixLock')
                refetchExerciseLoop()
              },
            })
          },
        })
        break
      }
    }

    asyncEffect()
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [dataExercise?.exerciseId?.running, dataExerciseLoop?.exerciseLoopRunning])

  return dataExerciseLoop?.exerciseLoopRunning
}

export default useExerciseSubscriptionStatus
